// funciones.cpp: Cdigo de las funciones de esta DLL
// ------------------------------------------------------------------

#include <stdafx.h>

#include "funciones.h"
#include <math.h>

// ------------------------------------------------------------------
// Funciones exportadas
// Estn reflejadas en el archivo .DEF del proyecto en el mismo orden
// ------------------------------------------------------------------

// OPI que pinta cruces sobre una imagen monocromo para representar
// las coordenadas de los blobs detectados con BlobsEx.dll

int Ejecutar(int iFunc, int iNumCtr, int iNumEnt, int iNumSal, CString& strDirTemp, 
				CString* pStrDatos, CString* pStrImgEntr, CString* pStrImgSal)
{
	// iFunc: debe ser 0
	// iNumCtr: debe ser 2
	// iNumEntr: debe ser 2
	// iNumSal: debe ser 1
	// strDirTemp: directorio temporal
	// pStrDatos[0]: Tamao del aspa
	// pStrDatos[1]: Color del aspa
	// pStrImgEntr[0]: imagen monocromo sobre la que pintar ejes
	// pStrImgEntr[1]: imagen float de N filas y 3 columnas con las coordenadas
	//				  y angulos de los N blobs a marcar:
	//				    X1 Y1 A1   X2 Y2 A2   ...   XN YN AN 
	// pStrImgSal[0]: imagen monocromo con las cruces pintadas sobre ellafloat de una fila y N+1 columnas con las coordenadas
	//				  y angulos de los blobs:
	
	int i;

	// Para depuracion
	#ifdef _DEBUG
	
		TRACE0("MODDLL: Modulo en ejecucion.\n");
		TRACE("Funcion: %d\nDatos: %d\nEntradas: %d\nSalidas: %d\n", 
			iFunc, iNumCtr, iNumEnt, iNumSal);
		TRACE("Dir. Temporal: %s\n", strDirTemp);
		TRACE0("Datos: ");
		for (i=0; i<iNumCtr; i++)
			TRACE("%d:%s\n", i, pStrDatos[i]);
		TRACE0("Entradas: ");
		for (i=0; i<iNumEnt; i++)
			TRACE("%d:%s\n", i, pStrImgEntr[i]);
		TRACE0("\nSalidas: ");
		for (i=0; i<iNumSal; i++)
			TRACE("%d:%s\n", i, pStrImgSal[i]);
		TRACE0("\n");
	#endif

	// Requiere dos imagenes de entrada, dos parmetros y 1 salida:
	if ((iNumEnt!=2) || (iNumCtr!=2) || (iNumSal!=1))
		return 1;

	// Lee valores de parametros de entrada
	int iTamano = atoi((LPCSTR)pStrDatos[0]);
	BYTE byColor = (BYTE)atoi((LPCSTR)pStrDatos[1]);

	// Crea y lee imagen de entrada
	StCabecera cabEnt;
	BYTE* pImgEntr = LeeImagenEntrada(strDirTemp + pStrImgEntr[0], 
							&cabEnt, 1);
	if (pImgEntr == NULL)
		return 2;

	// Crea y leer imagen con los blobs
	StCabecera cabBlobs;
	BYTE* pImgBlobs = LeeImagenEntrada(strDirTemp + pStrImgEntr[1], 
							&cabBlobs, sizeof(float));
	if (pImgEntr == NULL)
	{
		delete pImgEntr;
		return 3;
	}

	// Numero de blobs: filas de la imagen de blobs
	int iNumBlobs = (int)cabBlobs.wY;

	// Puntero paar acceder a los valores float de los blobs
	float* pImgBlobsF = (float*) pImgBlobs;

	// Coordenadas de un blob y angulo del eje
	int iPosX, iPosY;
	float fEje;

	// Tamao de la imagen en entero
	int iTamX = (int)cabEnt.wX;
	int iTamY = (int)cabEnt.wY;

	// Otras variables
	int minx, maxx, miny, maxy;
	double cose, sene;

	// Para conversind e grados a radianes
	double dPI180 = 3.14159265/180.0;
	
	// A pintar los blobs...
	for(int iBlob=0; iBlob<iNumBlobs; iBlob++)
	{
		// Obtiene coordenadas
		iPosX = (int)(*pImgBlobsF); pImgBlobsF++;
		iPosY = (int)(*pImgBlobsF); pImgBlobsF++;
		fEje = (*pImgBlobsF); pImgBlobsF++;

		// Si coordenadas no validas, las ignora
		if ((iPosX >= iTamX) || (iPosY >= iTamY))
			continue;
		
		// Calcula coordenadas para los ejes del aspa
		minx = iPosX - iTamano;
		if (minx<0) minx = 0;
		miny = iPosY - iTamano;
		if (miny<0) miny = 0;

		maxx = iPosX + iTamano;
		if (maxx>=iTamX) maxx = iTamX-1;
		maxy = iPosY + iTamano;
		if (maxy>=iTamY) maxy = iTamY-1;

		// "Pinta" aspa en el color indicado
		for(i=minx; i<=maxx; i++)
			pImgEntr[iPosY*iTamX+i] = byColor;

		for(i=miny; i<=maxy; i++)
			pImgEntr[i*iTamX+iPosX] = byColor;

		// Calcula eje del angulo
		cose = cos(fEje*dPI180);
		sene = sin(fEje*dPI180);
		for(i=-iTamano; i<iTamano; i++)
		{
			minx = iPosX + (int)(i*cose);
			miny = iPosY - (int)(i*sene);

			// Si esta en los lmites escribe el aspa
			if ((minx>=0) && (minx<iTamX) && (miny>=0) && (miny<iTamY))
				pImgEntr[miny*iTamX+minx] = byColor;
		}
	}

	// Escribe imagen de salida
	i = EscribeSalida(strDirTemp + pStrImgSal[0], &cabEnt, pImgEntr);
		
	// Guarda imagen de salida
	delete pImgEntr;
	delete pImgBlobs;
	
	// Libera memoria 
	return i;
}

// ------------------------------------------------------------------
// Funciones internas
// ------------------------------------------------------------------

// Leer imagen de entrada
BYTE* LeeImagenEntrada(CString& strNombre, StCabecera* pCab, WORD wByPP)
{
	CFile file;
	BYTE* pDatos;
	
	// Abrir la imagen de entrada
	if(!file.Open(strNombre, CFile::modeRead | CFile::shareDenyWrite))
		return NULL;

	// Leer cabecera de la imagen de entrada
	if (file.Read(pCab, sizeof(StCabecera)) < sizeof(StCabecera))
	{
		file.Close();
		return NULL;
	}

	// Debe tener almenos tamao 1 de ancho y de alto, y 1 canal, y ser
	// del tamao de pixel adecuado
	if ((pCab->wX < 1) || (pCab->wY < 1) || (pCab->wC < 1) || (pCab->wB != wByPP))
	{
		file.Close();
		return NULL;
	}

	// Calcula tamao de la imagen y reserva memoria
	DWORD dwTam = (DWORD)(pCab->wX) * (DWORD)pCab->wY * (DWORD)pCab->wC * (DWORD)pCab->wB;
	pDatos = new BYTE[dwTam];
	if (pDatos==NULL)
	{
		file.Close();
		return NULL;
	}
	
	// Leer datos de la imagen de entrada
	if (file.Read(pDatos, dwTam) < dwTam)
	{
		file.Close();
		return NULL;
	}
	
	// Cerrar archivo de entrada
	file.Close();

	return pDatos;
}

// Escribir imagen etiquetada se salida
int EscribeSalida(CString& strSalida, StCabecera* pCab, BYTE* pImgSal)
{	
	CFile file;
	int bOk=0;
		
	// Guarda salida
	if (file.Open(strSalida, CFile::modeCreate | CFile::modeWrite))
	{
		TRY
		{
			// Escribir cabecera
			file.Write(pCab, sizeof(StCabecera));

			// Calcula tamao de la imagen y reserva memoria
			DWORD dwTam = (DWORD)(pCab->wX) * (DWORD)pCab->wY * (DWORD)pCab->wC * (DWORD)pCab->wB;
				
			// Escribir contenido por filas
			file.Write(pImgSal, dwTam);
		}
		CATCH( CFileException, e )
		{
			bOk = 1;
		}
		END_CATCH
			
		file.Close();
	}
	else
		bOk = 1;

	return bOk;
}
